@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package Capture::Tiny;
 # ABSTRACT: Capture STDOUT and STDERR from Perl, XS or external programs
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 use Carp ();
 use Exporter ();
 use IO::Handle ();
@@ -48,12 +48,12 @@ our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
 
 my $IS_WIN32 = $^O eq 'MSWin32';
 
-#our $DEBUG = $ENV{PERL_CAPTURE_TINY_DEBUG};
-#
-#my $DEBUGFH;
-#open $DEBUGFH, "> DEBUG" if $DEBUG;
-#
-#*_debug = $DEBUG ? sub(@) { print {$DEBUGFH} @_ } : sub(){0};
+##our $DEBUG = $ENV{PERL_CAPTURE_TINY_DEBUG};
+##
+##my $DEBUGFH;
+##open $DEBUGFH, "> DEBUG" if $DEBUG;
+##
+##*_debug = $DEBUG ? sub(@) { print {$DEBUGFH} @_ } : sub(){0};
 
 our $TIMEOUT = 30;
 
@@ -63,7 +63,7 @@ our $TIMEOUT = 30;
 # This is annoying, but seems to be the best that can be done
 # as a simple, portable IPC technique
 #--------------------------------------------------------------------------#
-my @cmd = ($^X, '-e', '$SIG{HUP}=sub{exit}; '
+my @cmd = ($^X, '-C0', '-e', '$SIG{HUP}=sub{exit}; '
   . 'if( my $fn=shift ){ open my $fh, qq{>$fn}; print {$fh} $$; close $fh;} '
   . 'my $buf; while (sysread(STDIN, $buf, 2048)) { '
   . 'syswrite(STDOUT, $buf); syswrite(STDERR, $buf)}'
@@ -94,8 +94,8 @@ sub _open {
 }
 
 sub _close {
+  # _debug( "# closing " . ( defined $_[0] ? _name($_[0]) : 'undef' )  . " on " . fileno( $_[0] ) . "\n" );
   close $_[0] or Carp::confess "Error from close(" . join(q{, }, @_) . "): $!";
-  # _debug( "# closed " . ( defined $_[0] ? _name($_[0]) : 'undef' ) . "\n" );
 }
 
 my %dup; # cache this so STDIN stays fd0
@@ -124,7 +124,7 @@ sub _proxy_std {
     }
     else {
       _open \*STDOUT, ">" . File::Spec->devnull;
-      # _debug( "# proxied STDOUT as " . (defined fileno STDOUT ? fileno STDOUT : 'undef' ) . "\n" );
+       # _debug( "# proxied STDOUT as " . (defined fileno STDOUT ? fileno STDOUT : 'undef' ) . "\n" );
       _open $dup{stdout} = IO::Handle->new, ">&=STDOUT";
     }
     $proxies{stdout} = \*STDOUT;
@@ -134,11 +134,11 @@ sub _proxy_std {
     $proxy_count{stderr}++;
     if (defined $dup{stderr}) {
       _open \*STDERR, ">&=" . fileno($dup{stderr});
-      # _debug( "# restored proxy STDERR as " . (defined fileno STDERR ? fileno STDERR : 'undef' ) . "\n" );
+       # _debug( "# restored proxy STDERR as " . (defined fileno STDERR ? fileno STDERR : 'undef' ) . "\n" );
     }
     else {
       _open \*STDERR, ">" . File::Spec->devnull;
-      # _debug( "# proxied STDERR as " . (defined fileno STDERR ? fileno STDERR : 'undef' ) . "\n" );
+       # _debug( "# proxied STDERR as " . (defined fileno STDERR ? fileno STDERR : 'undef' ) . "\n" );
       _open $dup{stderr} = IO::Handle->new, ">&=STDERR";
     }
     $proxies{stderr} = \*STDERR;
@@ -149,7 +149,7 @@ sub _proxy_std {
 
 sub _unproxy {
   my (%proxies) = @_;
-  # _debug( "# unproxing " . join(" ", keys %proxies) . "\n" );
+  # _debug( "# unproxying: " . join(" ", keys %proxies) . "\n" );
   for my $p ( keys %proxies ) {
     $proxy_count{$p}--;
     # _debug( "# unproxied " . uc($p) . " ($proxy_count{$p} left)\n" );
@@ -272,7 +272,7 @@ sub _kill_tees {
 sub _slurp {
   my ($name, $stash) = @_;
   my ($fh, $pos) = map { $stash->{$_}{$name} } qw/capture pos/;
-  # _debug( "# slurping captured $name from $pos with layers: @{[PerlIO::get_layers($fh)]}\n");
+  # _debug( "# slurping captured $name from " . fileno($fh) . " at pos $pos with layers: @{[PerlIO::get_layers($fh)]}\n");
   seek( $fh, $pos, 0 ) or die "Couldn't seek on capture handle for $name\n";
   my $text = do { local $/; scalar readline $fh };
   return defined($text) ? $text : "";
@@ -332,8 +332,8 @@ sub _capture_tee {
   my %proxy_std = _proxy_std();
   # _debug( "# proxy std: @{ [%proxy_std] }\n" );
   # update layers after any proxying
-  $layers{stdout} = [PerlIO::get_layers(\*STDOUT)] if $proxy_std{stdout};
-  $layers{stderr} = [PerlIO::get_layers(\*STDERR)] if $proxy_std{stderr};
+  $layers{stdout} = [PerlIO::get_layers(\*STDOUT, output => 1)] if $proxy_std{stdout};
+  $layers{stderr} = [PerlIO::get_layers(\*STDERR, output => 1)] if $proxy_std{stderr};
   # _debug( "# post-proxy layers for $_\: @{$layers{$_}}\n" ) for qw/stdin stdout stderr/;
   # store old handles and setup handles for capture
   $stash->{old} = _copy_std();
@@ -411,20 +411,32 @@ Capture::Tiny - Capture STDOUT and STDERR from Perl, XS or external programs
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head1 SYNOPSIS
 
    use Capture::Tiny ':all';
  
+   # capture from external command
+ 
+   ($stdout, $stderr, $exit) = capture {
+     system( $cmd, @args );
+   };
+ 
+   # capture from arbitrary code (Perl or external)
+ 
    ($stdout, $stderr, @result) = capture {
      # your code here
    };
  
+   # capture partial or merged output
+ 
    $stdout = capture_stdout { ... };
    $stderr = capture_stderr { ... };
    $merged = capture_merged { ... };
  
+   # tee output
+ 
    ($stdout, $stderr) = tee {
      # your code here
    };
@@ -593,6 +605,9 @@ Note that this reopening will happen even for STDIN or a filehandle not being
 captured to ensure that the filehandle used for capture is not opened to file
 descriptor 0, as this causes problems on various platforms.
 
+Prior to Perl 5.12, closed STDIN combined with PERL_UNICODE=D leaks filehandles
+and also breaks tee() for undiagnosed reasons.  So don't do that.
+
 B<Localized filehandles>
 
 If code localizes any of Perl's standard filehandles before capturing, the capture
